package service

import (
	"go/ast"
)

var (
	isPointerType = false
)

func handleTypeSpec(s *ast.TypeSpec) (code string) {
	switch t := s.Type.(type) {
	case *ast.StructType:
		return handleClass(s.Name.String(), t)
	case *ast.InterfaceType:
		return handleClassAbstract(s.Name.String(), t)
	}

	return handleType(s)
}

func optionallyAddArrayDetails(s *ast.TypeSpec) (code string) {
	v, ok := s.Type.(*ast.ArrayType)
	if !ok {
		return
	}

	code += "["
	code += handleArrayLen(v)
	code += "]"
	return
}
func optionallyAddArrayDetailsArrayType(a *ast.ArrayType) (code string) {
	code += "["
	code += handleArrayLen(a)
	code += "]"
	return
}

func handleArrayLen(a *ast.ArrayType) (code string) {
	return handleExpr(a.Len)
}

func handleStructType(s *ast.StructType) (code string) {
	return handleFieldList(s.Fields)
}

func handleInterfaceType(s *ast.InterfaceType) (code string) {
	return handleFieldList(s.Methods)
}

func handleFieldList(l *ast.FieldList) string {
	code := ""
	for _, f := range l.List {
		code += handleField(f)
	}
	return code
}

func handleField(f *ast.Field) (code string) {
	// spew.Dump(f)
	code += "  "

	switch t := f.Type.(type) {
	case *ast.Ident:
		code += handleIdentExpr(t)
	case *ast.StarExpr:
		code += handleStarExpr(t)
	case *ast.FuncType:
		code += handleFuncDeclType(t)
	case *ast.ArrayType:
		code += handleArray(t)
	}

	if isClassByExpr(f.Type) {
		code += "*"
	}

	code += " "
	nado_zapyatuyu := false
	for _, n := range f.Names {
		if nado_zapyatuyu {
			code += ","
		}
		code += handleIdentExpr(n)
		nado_zapyatuyu = true
	}

	switch t := f.Type.(type) {
	case *ast.ArrayType:
		code += optionallyAddArrayDetailsArrayType(t)
	}

	code += ";\n"
	return
}

func isClassByExpr(e ast.Expr) (found bool) {
	name := getExprTypeName(e)
	for _, c := range classDeclarations {
		if c.Name() == name {
			found = true
			break
		}
	}
	return
}
func getExprTypeName(te ast.Expr) string {
	switch t := te.(type) {
	case *ast.Ident:
		return t.Name
	case *ast.StarExpr:
		ident, ok := t.X.(*ast.Ident)
		if ok {
			return ident.Name
		}
	}

	return ""
}
